#include <windows.h>
#include <string.h>
#include <stdlib.h>
#include <assert.h>
#include "registry.h"

static BOOL rquery_value( const REG* rp, const char* value_name, DWORD dwType, void* buffer, DWORD buffer_size );
static BOOL rset_value( REG* rp, const char* value_name, DWORD dwType, const BYTE* pData, DWORD dwBytes );

REG* ropen( HKEY hKeyOpened, const char* szRegistryKey, const char* mode )
{
	if( hKeyOpened == NULL ) return NULL;
	REGSAM samDesired = NULL;
	if( mode != NULL )
	{
		if( strchr(mode,'r') != NULL ) samDesired |= KEY_READ;
		if( strchr(mode,'w') != NULL ) samDesired |= KEY_WRITE;
		if( strchr(mode,'l') != NULL ) samDesired |= KEY_CREATE_LINK;
		if( strchr(mode,'a') != NULL )
			samDesired |= KEY_ALL_ACCESS;
		if( strstr(mode,"r+") != NULL || strstr(mode,"w+") != NULL )
			samDesired |= (KEY_READ | KEY_WRITE);
		if( strstr(mode,"l+") != NULL )
			samDesired |= KEY_ALL_ACCESS;
	}
	else
	{
		samDesired = KEY_ALL_ACCESS;
	}
	if( samDesired == NULL ) return NULL;
	
	HKEY hKey;
	DWORD dwDisposition;
	LONG lResult = RegCreateKeyEx( hKeyOpened, szRegistryKey, NULL, "",
						REG_OPTION_NON_VOLATILE, samDesired, NULL, &hKey, &dwDisposition );
	if( lResult != ERROR_SUCCESS ) return NULL;
	REG* ret = new REG;
	if( ret == NULL ) return NULL;
	ret->hKey = hKey;
	ret->dwDisposition = dwDisposition;
	return ret;
}

REG* ropen( const char* szRegistryKey, const char* mode )
{
	return ropen( HKEY_CURRENT_USER, szRegistryKey, mode );
}

BOOL rclose( REG* rp )
{
	if( rp == NULL ) return TRUE;
	LONG lResult = RegCloseKey( rp->hKey );
	if( lResult != ERROR_SUCCESS ) return FALSE;
	delete rp;
	return TRUE;
}

BOOL rdelete( const REG* rp, const char* subkey )
{
	if( rp == NULL || subkey == NULL ) return FALSE;
	LONG lResult = RegDeleteKey( rp->hKey, subkey );
	if( lResult != ERROR_SUCCESS ) return FALSE;
	return TRUE;
}

BOOL rdelete_value( const REG* rp, const char* value_name )
{
	if( rp == NULL || value_name == NULL ) return FALSE;
	LONG lResult = RegDeleteValue( rp->hKey, value_name );
	if( lResult != ERROR_SUCCESS ) return FALSE;
	return TRUE;
}

void rflush( const REG* rp )
{
	if( rp == NULL ) return;
	RegFlushKey( rp->hKey );
	return;
}

static BOOL rquery_value( const REG* rp, const char* value_name, DWORD dwType, void* buffer, DWORD buffer_size )
{
	if( rp == NULL || value_name == NULL ) return FALSE;
	DWORD dwTypeReceive;
	LONG lResult;
	if( value_name != NULL )
	{
		lResult = RegQueryValueEx( rp->hKey, value_name, NULL, &dwTypeReceive,
													(BYTE*)buffer, &buffer_size );
	}
	else
	{
		lResult = RegQueryValue( rp->hKey, NULL, (LPTSTR)buffer, (LONG*)&buffer_size );
	}
	if( lResult != ERROR_SUCCESS ) return FALSE;
	if( (dwTypeReceive & dwType) == NULL ) return FALSE;
	return TRUE;
}

int rget_valuesize( const REG* rp, const char* value_name )
{
	if( rp == NULL ) return -1;
	DWORD size;
	LONG lResult;
	if( value_name != NULL )
	{
		lResult = RegQueryValueEx( rp->hKey, value_name, NULL, NULL, NULL, &size );
	}
	else
	{
		lResult = RegQueryValue( rp->hKey, NULL, NULL, (LONG*)&size );
	}
	if( lResult != ERROR_SUCCESS ) return -1;
	return (int)size;
}

const char* rgets( char* buffer, int size_buffer, const REG* rp, const char* value_name )
{
	if( buffer == NULL || rp == NULL ) return NULL;
	BOOL r = rquery_value( rp, value_name, REG_SZ, buffer, size_buffer );
	if( r == FALSE ) return NULL;
	return buffer;
}

int rgeti( const REG* rp, const char* value_name, int default_value )
{
	if( rp == NULL ) return default_value;
	if( value_name == NULL )
	{
		char sz_value[32];
		const char* r = rgets( sz_value, 32, rp, NULL );
		if( r == NULL ) return default_value;
		return atoi( sz_value );
	}
	DWORD dwReceive;
	BOOL r = rquery_value( rp, value_name, REG_DWORD, &dwReceive, sizeof(DWORD) );
	if( r == FALSE ) return default_value;
	return dwReceive;
}

static BOOL rset_value( REG* rp, const char* value_name, DWORD dwType, const BYTE* pData, DWORD dwBytes )
{
	if( value_name == NULL ) dwType = REG_SZ;
	LONG lResult = RegSetValueEx( rp->hKey, value_name, 0, dwType, pData, dwBytes );
	if( lResult != ERROR_SUCCESS ) return FALSE;
	return TRUE;
}

int rputs( const char* string, REG* rp, const char* value_name )
{
	if( rp == NULL ) return -1;
	int l = lstrlen( string );
	BOOL r = rset_value( rp, value_name, REG_SZ, (BYTE*)string, l + 1 );
	if( r == FALSE ) return -1;
	return l + 1;
}

int rputi( int value, REG* rp, const char* value_name )
{
	if( value == NULL ) return -1;
	BOOL r;
	if( value_name == NULL )
	{
		char sz_value[32];
		wsprintf( sz_value, "%d", value );
		r = rputs( sz_value, rp, NULL );
	}
	else
	{
		r = rset_value( rp, value_name, REG_DWORD, (const BYTE*)&value, sizeof(DWORD) );
	}
	if( r == FALSE ) return -1;
	return sizeof(DWORD);
}
